package com.gic.cloud.data.hook.service;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import com.alibaba.fastjson.JSONObject;
import com.ctrip.framework.apollo.Config;
import com.ctrip.framework.apollo.ConfigService;
import com.gic.cloud.data.hook.api.entity.FlatQueryCondition;
import com.gic.cloud.data.hook.api.entity.FlatQueryFieldType;
import com.gic.cloud.data.hook.service.entity.TimeCheckPojo;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.concurrent.BasicThreadFactory;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import java.util.*;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

@Component
public class FilterFieldUtils {

    private static Logger logger= LoggerFactory.getLogger(FilterFieldUtils.class);

    /** 脱敏字段 */
    public static List<String> FILTERS_PHONE_ONLY = Arrays.asList("mobile", "phone", "enterprise_name", "phone_number", "receive_phone_number", "use_phone_number", "friend_phone_num","from_phone_num");

    /** 脱敏字段 */
    public static List<String> FILTERS_PHONE_AND_CARD = Arrays.asList("card_num", "mobile", "phone", "enterprise_name", "phone_number", "receive_phone_number","receive_card_num","use_phone_number", "use_card_num","friend_card_num","from_card_num", "friend_phone_num","from_phone_num");

    /** 脱敏字段 */
    public static List<String> FILTERS_PHONE_AND_CARD_COUPON = Arrays.asList("card_num", "mobile", "phone", "enterprise_name", "phone_number", "receive_phone_number","receive_card_num","use_phone_number", "use_card_num","friend_card_num","from_card_num", "friend_phone_num","from_phone_num","card_code");

    public static List<String> FILETERS_USER_NAME = Arrays.asList("member_name","children_name","mbr_name","receive_member_name","use_member_name","name","bb_name","friend_mbr_name","from_mbr_name","card_code");

    ScheduledExecutorService timer = new ScheduledThreadPoolExecutor(1,
            new BasicThreadFactory.Builder().namingPattern("loadFilterFieldTimer-%d").daemon(true).build());

    @PostConstruct
    public void init(){
        initTimer();
    }

    private void initTimer(){
        timer.scheduleAtFixedRate(new Runnable() {
            @Override
            public void run() {
                Config config = ConfigService.getConfig("application");
                String phoneNumberField = config.getProperty("phoneNumberField", "");
                String cardNumField = config.getProperty("cardNumField", "");
                String userNameField = config.getProperty("userNameField", "");
                if(StringUtils.isNotBlank(phoneNumberField)){
                    FILTERS_PHONE_ONLY = Arrays.asList(phoneNumberField.split(","));
                    FILTERS_PHONE_AND_CARD = new ArrayList<>();
                    for(String s : phoneNumberField.split(",")){
                        FILTERS_PHONE_AND_CARD.add(s);
                    }
                }
                if(StringUtils.isNotBlank(cardNumField)){
                    for(String s : cardNumField.split(",")){
                        FILTERS_PHONE_AND_CARD.add(s);
                    }
                }
                if(StringUtils.isNotBlank(userNameField)){
                    FILETERS_USER_NAME = Arrays.asList(userNameField.split(","));
                }
            }
        }, 5, 5, TimeUnit.SECONDS);
    }


    private static ThreadLocal<Map<String,TimeCheckPojo>> TIMECHECK_THREADLOCAL=new ThreadLocal<>();

    /**
     * 初始化 时间校验
     * @param queryConditions
     */
    public static void timeFieldCheckInit(List<FlatQueryCondition> queryConditions) {
        TIMECHECK_THREADLOCAL.remove();
        HashMap<String,TimeCheckPojo> retMap=new HashMap<>();
        if (CollectionUtils.isEmpty(queryConditions)) {
            return;
        }
        for (FlatQueryCondition queryCondition : queryConditions) {
            // 如果存在主过滤器的值，视为过滤器生效
            if (StringUtils.isNotBlank(queryCondition.getMainFilter())
                    &&queryCondition.getFieldType().equals(FlatQueryFieldType.DATE)) {
                String startDate = queryCondition.getMainFilter() + "";
                String overDate = queryCondition.getExtendFilter() + "";
                DateTime start = DateUtil.parse(startDate);
                DateTime end = DateUtil.parse(overDate);

                TimeCheckPojo timeCheckPojo=new TimeCheckPojo();
                timeCheckPojo.setEndStr(overDate);
                timeCheckPojo.setStartStr(startDate);
                timeCheckPojo.setEndDate(end);
                timeCheckPojo.setStartDate(start);
                retMap.put(queryCondition.getFieldName(),timeCheckPojo);
            }
        }
        logger.info("开始----：{}",JSONObject.toJSONString(retMap));
        TIMECHECK_THREADLOCAL.set(retMap);
    }

    /**
     * 校验时间
     * @param key
     * @param time
     * @return
     */
    public static void checkTime(String key,String time,Object orgObj){
        Map<String, TimeCheckPojo> checkPojoMap = TIMECHECK_THREADLOCAL.get();
        if (checkPojoMap == null) {
            return;
        }
        TimeCheckPojo timeCheckPojo = checkPojoMap.get(key);
        if (timeCheckPojo == null) {
            return ;
        }
        DateTime parse = DateUtil.parse(time);
        if(parse.isAfter(timeCheckPojo.getStartDate())&&parse.isBefore(timeCheckPojo.getEndDate())){
            return ;
        }
        logger.info("key:{},time:{}---{},,orgTime:{}",key,time,orgObj, JSONObject.toJSON(timeCheckPojo));
    }

    /**
     * 是否校验
     */
    public static void removeTimeCheck(){
        logger.info("结束----");
        TIMECHECK_THREADLOCAL.remove();
    }
}
