package com.gic.cloud.data.hook.service;

import com.ctrip.framework.apollo.Config;
import com.ctrip.framework.apollo.ConfigService;
import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.fs.FileSystem;
import org.apache.hadoop.fs.Path;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import java.net.URI;

public class HDFSUtil {
    private static Logger logger = LoggerFactory.getLogger(HDFSUtil.class);
    private static HDFSUtil hdfsUtil=null;
    private static FileSystem fileSystem=null;

    public static HDFSUtil getInstance(){
        if (hdfsUtil==null) {
            synchronized (HDFSUtil.class) {
                if (hdfsUtil==null) {
                    hdfsUtil = new HDFSUtil();
                }
            }
        }
        return hdfsUtil;
    }

    private HDFSUtil(){
        Config appConfig = ConfigService.getAppConfig();
        String hdfsUrl = appConfig.getProperty("hdfs.url", null);
        Configuration configuration = new Configuration();
        configuration.set("fs.defaultFS",hdfsUrl);
        try {
            fileSystem = FileSystem.get(new URI(hdfsUrl),configuration,"hadoop");
        } catch (Exception e) {
            logger.info("hdfs初始化失败-{}",e);
        }
    }

    /**
     * 下载到本地
     * @param srcPath
     * @param toPath
     * @return
     */
    public boolean downloadFile(String srcPath,String toPath) {
        try {
            fileSystem.copyToLocalFile(false,new Path(srcPath),new Path(toPath));
            return true;
        } catch (IOException e) {
            logger.info("下载失败：{}",e);
            return false;
        }
    }
}
