package com.gic.cloud.data.hook.service.entity;

import cn.hutool.core.codec.Base64;
import cn.hutool.core.util.CharsetUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.crypto.asymmetric.KeyType;
import cn.hutool.crypto.asymmetric.RSA;
import com.alibaba.dubbo.common.utils.StringUtils;
import com.ctrip.framework.apollo.Config;
import com.ctrip.framework.apollo.ConfigService;
import com.gic.cloud.data.hook.api.dto.DhDecryptKey;
import com.gic.cloud.data.hook.api.entity.FlatQueryCondition;
import com.gic.cloud.data.hook.api.service.DecryptKeyService;
import com.gic.cloud.data.hook.service.DecryptUtils;
import com.gic.cloud.data.hook.service.FilterFieldUtils;
import com.opencsv.ResultSetHelper;
import org.apache.commons.collections.CollectionUtils;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

public class CsvResultSetHelper implements ResultSetHelper {

	/** 数据过滤模式 */
	private String filterMode = CsvDataFilterMode.NONE;

	/** 数据过滤用关键字集合 */
	private List<String> filters = null;

	private List<FlatQueryCondition> allFields = null;
	private String enterpriseId;
	private List<String> decryptEnterpriseIdList = new ArrayList<>();
	private boolean needDecryptByEnt = false;
	private DecryptKeyService decryptKeyService;
	private String publicKey;
	private RSA rsa;


	public CsvResultSetHelper(String filterMode, List<String> filters, List<FlatQueryCondition> allFields,
							  String enterpriseId, DecryptKeyService decryptKeyService) {
		this.filterMode = filterMode;
		this.filters = filters;
		this.allFields = allFields;
		this.enterpriseId = enterpriseId;
		this.decryptKeyService = decryptKeyService;
		Config config = ConfigService.getConfig("application");
		String decryptEnterpriseIds = config.getProperty("decryptEnterpriseIds", "");
		if(org.apache.commons.lang.StringUtils.isNotBlank(decryptEnterpriseIds) && allFields != null){
			decryptEnterpriseIdList = Arrays.asList(decryptEnterpriseIds.split(" "));
			if(decryptEnterpriseIdList.contains(enterpriseId)){
				needDecryptByEnt = true;
				//创建密钥
				DhDecryptKey key = decryptKeyService.getKeyByEnterpriseId(enterpriseId);
				rsa = new RSA();
				if(key == null){
					decryptKeyService.saveDecryKey(enterpriseId, rsa.getPublicKeyBase64(), rsa.getPrivateKeyBase64());
					publicKey = rsa.getPublicKeyBase64();
				} else {
					publicKey = key.getPublicKey();
					rsa = new RSA(key.getPrivateKey(), key.getPublicKey());
				}
			}
		}
	}

	@Override
	public String[] getColumnNames(ResultSet resultSet) throws SQLException {
		List<String> list = new ArrayList<>();
		int columnCount = resultSet.getMetaData().getColumnCount();
		if (columnCount > 0) {
			for (int i=0; i<columnCount; i++) {
				String rsColumnName = resultSet.getMetaData().getColumnLabel(i + 1);
				list.add(rsColumnName);
			}
		}
		if(needDecryptByEnt){  //补齐加密字段
			for(FlatQueryCondition condition : allFields){
				if(FilterFieldUtils.FILTERS_PHONE_ONLY.contains(condition.getFieldName())){
					list.add(condition.getFieldMark());
				}
			}
		}
		return list.toArray(new String[0]);

	}

	@Override
	public String[] getColumnValues(ResultSet resultSet) throws SQLException, IOException {
		return this.getColumnValues(resultSet, false);
	}

	@Override
	public String[] getColumnValues(ResultSet resultSet, boolean b) throws SQLException, IOException {
		return this.getColumnValues(resultSet, false, "yyyy-MM-dd HH:mm:ss", "HH:mm:ss");
	}


	private static SimpleDateFormat datetimeFormatter = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

	private static ThreadLocal<DateFormat> datetimeFormatter_threadLocal = new ThreadLocal<DateFormat>() {
		@Override
		protected DateFormat initialValue() {
			return new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
		}
	};

	private static SimpleDateFormat dateFormatter = new SimpleDateFormat("yyyy-MM-dd");

	private static ThreadLocal<DateFormat> dateFormatter_threadLocal = new ThreadLocal<DateFormat>() {
		@Override
		protected DateFormat initialValue() {
			return new SimpleDateFormat("yyyy-MM-dd");
		}
	};

	private static SimpleDateFormat timeFormatter = new SimpleDateFormat("hh:mm:ss");
	private static ThreadLocal<DateFormat> timeFormatter_threadLocal = new ThreadLocal<DateFormat>() {
		@Override
		protected DateFormat initialValue() {
			return new SimpleDateFormat("hh:mm:ss");
		}
	};

	@Override
	public String[] getColumnValues(ResultSet resultSet, boolean b, String s, String s1) throws SQLException, IOException {
		List<String> result = new ArrayList<>();
		List<String> decyptValue = new ArrayList<>();
		int columnCount = resultSet.getMetaData().getColumnCount();
		if (columnCount > 0) {
			for (int i=0; i<columnCount; i++) {
				String columnName = resultSet.getMetaData().getColumnName(i + 1);
				// 数据处理判断
				boolean doDesensi = false; // 是否进行脱敏
				boolean doDecrypt = false; // 是否进行解密
				if (this.filterMode.equals(CsvDataFilterMode.DESENSI)) { // 如果需要脱敏
					if (this.filters != null && this.filters.size() > 0) {
						if(filters.contains(columnName)){
							doDesensi = true;
						}
					}
					if(allFields != null && isName(columnName)){
						doDesensi = true;
					}
					if(allFields == null && FilterFieldUtils.FILETERS_USER_NAME.contains(columnName)){
						doDesensi = true;
					}
				} else if (this.filterMode.equals(CsvDataFilterMode.DECRYPT)) { // 如果需要解密
					if (this.filters != null && this.filters.size() > 0) {
						if(filters.contains(columnName)){
							doDecrypt = true;
						}
					}
					if(allFields != null && isName(columnName)){
						doDecrypt = true;
					}
					if(allFields == null && FilterFieldUtils.FILETERS_USER_NAME.contains(columnName)){
						doDecrypt = true;
					}
				}
				System.out.println("doDecrypt = " + doDecrypt);
				// 数据处理
				if (doDesensi) { // 如果需要脱敏处理
					if(allFields != null && isName(columnName)){
						result.add(DecryptUtils.dataSecurityProcessUserName(DecryptUtils.decrypt(enterpriseId,resultSet.getString(columnName))));
					} else if(allFields == null && FilterFieldUtils.FILETERS_USER_NAME.contains(columnName)) {
						result.add(DecryptUtils.dataSecurityProcessUserName(resultSet.getString(i+1)));
					} else {
						result.add("******");
						if(needDecryptByEnt && isPhoneNumber(columnName)){
							//导出数据中需要增加加密列,先解密再加密
							String value = DecryptUtils.getInstance().decrypt(enterpriseId,resultSet.getString(i+1));
							if(org.apache.commons.lang3.StringUtils.isNotBlank(value)){
								byte[] bytes = rsa.encrypt(StrUtil.bytes(value, CharsetUtil.UTF_8), KeyType.PublicKey);
								String encode = Base64.encode(bytes);
								decyptValue.add(encode);
							} else {
								decyptValue.add("");
							}
						}
					}
				} else if (doDecrypt) { // 如果需要解密处理
					String tmpResult = resultSet.getString(i+1);
					if (tmpResult != null && tmpResult.length() > 0) {
						//tmpResult = DecryptUtils.getInstance().decrypt(tmpResult);
						tmpResult = DecryptUtils.getInstance().decrypt(enterpriseId,tmpResult);
						if(tmpResult.startsWith("0")){
							tmpResult = tmpResult + "\t";
						}
						System.out.println("tmpResult = " + tmpResult);
					} // IF OVER
					result.add(tmpResult);
				} else {
					int columnType = resultSet.getMetaData().getColumnType(i + 1);
					switch (columnType) {
						case Types.TIMESTAMP:
							String time = resultSet.getTimestamp(i+1) != null ? datetimeFormatter_threadLocal.get().format(resultSet.getTimestamp(i+1)) : "";
							result.add(time);
//							FilterFieldUtils.checkTime(columnName,time,resultSet.getObject(i+1));
							break;
						case Types.DATE:
							//result[i] = resultSet.getTimestamp(columnName) != null ? dateFormatter.format(resultSet.getTimestamp(columnName)) : "";
							result.add(resultSet.getDate(i+1) != null ? dateFormatter_threadLocal.get().format(resultSet.getDate(i+1)) : "");
							break;
						case Types.TIME:
							result.add(resultSet.getTimestamp(i+1) != null ? timeFormatter_threadLocal.get().format(resultSet.getTimestamp(i+1)) : "");
							break;
						default:
//							String s2 = String.valueOf(resultSet.getObject(i + 1));
							Object val = resultSet.getObject(i + 1);
							String s2="";
							if (val != null) {
								s2 = String.valueOf(val);
							}
//							if(org.apache.commons.lang3.StringUtils.isNotBlank(s2)){
//								if(s2.contains("E0") || s2.startsWith("0")){
//									s2 = s2 + "\t";
//								}
//								s2=s2.replace("\r","");
//								s2=s2.replace("\n","");
//								s2=s2.replace(",","，");
//								s2=s2.replace("\"","");
//							}
							result.add(s2);
							break;
					} // SWITCH OVER
				} // IF ELSE OVER
			} // FOR OVER
		} // IF OVER
		if(CollectionUtils.isNotEmpty(decyptValue)){
			result.addAll(decyptValue);
		}
		return result.toArray(new String[0]);
	}

	private boolean isName(String columnName){
		if(StringUtils.isBlank(columnName)){
			return false;
		}
		for(FlatQueryCondition condition : allFields){
			if(condition.getFieldMark().equals(columnName)){
				if(FilterFieldUtils.FILETERS_USER_NAME.contains(condition.getFieldName())){
					return true;
				}
			}
		}
		return false;
	}

	private boolean isPhoneNumber(String columnName){
		if(StringUtils.isBlank(columnName)){
			return false;
		}
		for(FlatQueryCondition condition : allFields){
			if(condition.getFieldMark().equals(columnName)){
				if(FilterFieldUtils.FILTERS_PHONE_ONLY.contains(condition.getFieldName())){
					return true;
				}
			}
		}
		return false;
	}

	public static void main(String[] args) throws UnsupportedEncodingException {
		RSA rsa = new RSA();
		String value = "18989474184";
		byte[] s = rsa.encrypt(StrUtil.bytes(value, CharsetUtil.UTF_8), KeyType.PublicKey);
		String encode = Base64.encode(s);
		System.out.println("encode = " + encode);
		byte[] decode = Base64.decode(encode.getBytes());
		String s1 = new String(rsa.decrypt(decode, KeyType.PrivateKey));
		System.out.println("s1 = " + s1);
	}
}
