package com.gic.cloud.data.hook.service.impl;

import cn.medubi.client.utils.LogPak;
import com.alibaba.fastjson.JSON;
import com.ctrip.framework.apollo.Config;
import com.ctrip.framework.apollo.ConfigService;
import com.gic.cloud.common.api.base.Page;
import com.gic.cloud.data.hook.api.dto.*;
import com.gic.cloud.data.hook.api.entity.DownloadTaskStatus;
import com.gic.cloud.data.hook.api.entity.FreeQueryTaskCondition;
import com.gic.cloud.data.hook.api.entity.Global;
import com.gic.cloud.data.hook.api.service.IDownloadTaskService;
import com.gic.cloud.data.hook.service.MysqlHelper;
import com.gic.cloud.data.hook.service.dao.DownloadRecordDao;
import com.gic.cloud.data.hook.service.dao.DownloadTaskDao;
import com.gic.cloud.data.hook.service.dao.RiskModeRecordDao;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.opencsv.CSVWriter;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.concurrent.BasicThreadFactory;
import org.apache.ibatis.annotations.Param;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.stereotype.Service;

import java.io.*;
import java.nio.charset.Charset;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.*;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledThreadPoolExecutor;
import java.util.concurrent.TimeUnit;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

/** 下载任务服务即成
 * @author Sam
 */
@Component
public class DownloadTaskServiceImpl implements IDownloadTaskService {

	/** 日志类 */
	private static LogPak log = new LogPak(DownloadTaskServiceImpl.class);

	/** 下载任务服务单例 */
	private static DownloadTaskServiceImpl instance = null;

	/** 获取下载任务服务单例
	 * @return
	 */
	public static synchronized DownloadTaskServiceImpl getInstance() {
		if (instance == null) instance = new DownloadTaskServiceImpl();
		return instance;
	}

	/** 构造函数 */
	private DownloadTaskServiceImpl() {
		log.debug("construct", "准备初始化 DownloadTask 查询服务");
		instance = this;
		this.runDownloadCleanTask(60); // 每分钟清理一次
	}

	@Autowired
	private DownloadTaskDao downloadTaskDao;

	@Autowired
	private DownloadRecordDao downloadRecordDao;

	@Autowired
	private RiskModeRecordDao riskModeRecordDao;

	/** 获取指定的下载任务
	 * @param taskId
	 * @return
	 */
	public DownloadTask getDownloadTaskById(String taskId) {
		return this.downloadTaskDao.getDownloadTaskById(taskId);
	}

	/** 更新指定的下载任务
	 * @param task
	 * @return
	 */
	public Integer updateDownloadTask(DownloadTask task) {
		return this.downloadTaskDao.updateDownloadTask(task);
	}

	@Override
	public Page<DownloadTask> getDownloadTaskPage(String userId, String fuzzy, Integer pageNum, Integer pageSize) {
		checkTaskStatus("");
		Page<DownloadTask> result = new Page<>();
		PageHelper.startPage(pageNum, pageSize); // 设置分页参数
		List<DownloadTask> query = downloadTaskDao.getDownloadTaskListByUserId(userId, fuzzy);
		// 生成数据输出信息
		PageInfo<DownloadTask> preResult = new PageInfo<>(query);
		result.setList(preResult.getList()); // 设置数据集
		result.setTotal(preResult.getTotal()); // 设置总数
		result.setPageNum(preResult.getPageNum()); // 设置当前页数

		if (CollectionUtils.isNotEmpty(result.getList())) {
			for (DownloadTask downloadTask : result.getList()) {
				if (StringUtils.isBlank(downloadTask.getFilePath())) {
					continue;
				}
				downloadTask.setFilePath(getDownloadUrl(downloadTask.getFilePath(),downloadTask.getName()));
			}
		}
		return result;
	}

	private String getDownloadUrl(String url,String filename) {
		Config config = ConfigService.getConfig("COMMON.cloud.file.setting");
		String hostUrl = config.getProperty("gic.proxy.url", null);
		if (org.apache.commons.lang.StringUtils.isBlank(hostUrl)) {
			return url;
		}else{
			int pos = url.lastIndexOf(".");
			String ext = url.substring(pos);
			return hostUrl+"/"+filename+ext+"?url="+url;
		}
	}

	/** 创建下载任务
	 * @param task
	 * @return
	 */
	public Integer createDownloadTask(DownloadTask task) {
		return this.downloadTaskDao.createDownloadTask(task);
	}


	/** 删除指定的下载任务
	 * @param taskId
	 * @return
	 */
	public Boolean deleteDownloadTask(String taskId) {
		DownloadTask task = this.downloadTaskDao.getDownloadTaskById(taskId);
		if (task != null) {
			if (task.getStatus().equals(DownloadTaskStatus.COMPLISHED) || task.getStatus().equals(DownloadTaskStatus.ERROR)) {
				if (task.getFormat().equals("csv")) {
					File csv = new File("/usr/local/data-hook-file/" + task.getId() + ".csv");
					if (csv.exists()) csv.delete();
					if (task.getUseCompress().equals(Global.YES)) {
						File zip = new File("/usr/local/data-hook-file/" + task.getId() + ".zip");
						if (zip.exists()) zip.delete();
					}
				} else {
					File xlsx = new File("/usr/local/data-hook-file/" + task.getId() + ".xlsx");
					if (xlsx.exists()) xlsx.delete();
					if (task.getUseCompress().equals(Global.YES)) {
						File zip = new File("/usr/local/data-hook-file/" + task.getId() + ".zip");
						if (zip.exists()) zip.delete();
					}
				}
			}
			this.downloadTaskDao.deleteDownloadTask(task.getId());
			return true;
		} else return false;
	}

	/** 获取指定企业的所有下载任务编号
	 * @param enterpriseId
	 * @return
	 */
	public Integer getDownloadTaskCountByEnterpriseId(String enterpriseId) {
		return this.downloadTaskDao.countDownloadTaskByEnterpriseId(enterpriseId);
	}

	/** 下载任务执行计时器 */
	//private Timer downloadTaskTimer = new Timer();
	ScheduledExecutorService deleteExpireTaskService = new ScheduledThreadPoolExecutor(1,
			new BasicThreadFactory.Builder().namingPattern("deleteExpireTask-%d").daemon(true).build());

	/** 启动自动清理下载任务服务 */
	private void runDownloadCleanTask(Integer interval) {
		deleteExpireTaskService.scheduleAtFixedRate(new Runnable() {
			@Override
			public void run() {
				Calendar calc = Calendar.getInstance();
				//calc.add(Calendar.DATE, -1);
				calc.add(Calendar.HOUR, -48); // 从一天改为 12 小时
				Date expireDate = calc.getTime();
				List<DownloadTask> expiredTasks = downloadTaskDao.getExpiredDownloadTaskList(expireDate);
				if (expiredTasks.size() > 0) {
					for (DownloadTask task : expiredTasks) {
						deleteDownloadTask(task.getId());
					} // 遍历删除
				} // IF OVER
			}
		}, interval*1000,interval*1000, TimeUnit.MILLISECONDS);
	}


	@Override
	public Integer createDownloadRecord(String taskId, String userId, String userName, String enterpriseId) {
		DownloadRecord record = new DownloadRecord();
		record.setId(String.valueOf((new Date()).getTime()));
		record.setDownloadTime(new Date());
		record.setTaskId(taskId);
		record.setUserId(userId);
		record.setUserName(userName);
		record.setEnterpriseId(enterpriseId);
		return this.downloadRecordDao.createDownloadRecord(record);
	}

	@Override
	public Page<DownloadRecord> getDownloadRecordPage(String userId, String enterpriseId, Date startTime, Date endTime, Integer queryDataType, String fuzzyRange, String fuzzy, Integer pageNum, Integer pageSize) {
		Page<DownloadRecord> result = new Page<>();
		PageHelper.startPage(pageNum, pageSize); // 设置分页参数
		List<DownloadRecord> query = downloadRecordDao.getDownloadRecordList(userId, enterpriseId, startTime, endTime, queryDataType, fuzzyRange, fuzzy);
		// 生成数据输出信息
		PageInfo<DownloadRecord> preResult = new PageInfo<>(query);
		result.setList(preResult.getList()); // 设置数据集
		result.setTotal(preResult.getTotal()); // 设置总数
		result.setPageNum(preResult.getPageNum()); // 设置当前页数
		return result;
	}

	/** 获取在审核申请等待状态中的任务
	 * @return
	 */
	public List<DownloadTask> getDownloadTaskOfWaiting(String queryDataSource) {
		return downloadTaskDao.getDownloadTaskOfWaiting(queryDataSource);
	}

	/** 获取在审核申请等待状态中的任务
	 * @return
	 */
	public List<DownloadTask> getDownloadTaskOfHasDownload(String queryDataSource,int num) {
		return downloadTaskDao.getDownloadTaskOfHasDownload(queryDataSource, num);
	}


	/** 获取指定申请编号的风险模式记录
	 * @param applyId
	 * @return
	 */
	public RiskModeRecord getRiskModeRecordByApplyId(String applyId) {
		if (StringUtils.isNotBlank(applyId)) {
			return this.riskModeRecordDao.getRiskModeRecordByApplyId(applyId);
		} else return null;
	}

	/** 创建风险模式记录
	 * @param riskModeRecord
	 */
	@Override
	public void createRiskModeRecord(RiskModeRecord riskModeRecord) {
		String applyId = riskModeRecord.getApplyId();
		if (StringUtils.isNotBlank(applyId)) {
			RiskModeRecord preRecord = this.getRiskModeRecordByApplyId(applyId);
			if (preRecord == null) {
				riskModeRecord.setCreateTime(new Date());
				this.riskModeRecordDao.createRiskModeRecord(riskModeRecord);
			} else log.debug("createRiskModeRecord", "指定编号为 " + applyId + " 的风险模式记录已存在，无法创建");
		} else log.debug("createRiskModeRecord", "指定的风险模式编号不能为空");
	}

	/** 更新风险模式切换日志
	 * @param riskModeRecord
	 */
	public void updateRiskModeRecord(RiskModeRecord riskModeRecord) {
		this.riskModeRecordDao.updateRiskModeRecord(riskModeRecord);
	}

	@Override
	public Page<RiskModeRecord> getRiskModeRecordPage(String enterpriseId, Date startTime, Date endTime, Integer riskDuration, String fuzzyRange, String fuzzy, Integer pageNum, Integer pageSize) {
		Page<RiskModeRecord> result = new Page<>();
		PageHelper.startPage(pageNum, pageSize); // 设置分页参数
		List<RiskModeRecord> query = riskModeRecordDao.getRiskModeRecordList(enterpriseId, startTime, endTime, riskDuration, fuzzyRange, fuzzy);
		// 生成数据输出信息
		PageInfo<RiskModeRecord> preResult = new PageInfo<>(query);
		result.setList(preResult.getList()); // 设置数据集
		result.setTotal(preResult.getTotal()); // 设置总数
		result.setPageNum(preResult.getPageNum()); // 设置当前页数
		return result;
	}

	@Override
	public List<DownloadRecord> listUnDownloadTask(String source) {
		return this.downloadRecordDao.listUnDownloadTask(source);
	}

	@Override
	public DownloadProcessDTO getDownloadProcess() {
		DownloadProcessDTO downloadProcess = this.downloadTaskDao.getDownloadProcess();
		return downloadProcess;
	}

	@Override
	public void checkTaskStatus(String param) {
		System.out.println("执行定时任务");
		List<DownloadRecord> downloadRecordList = this.downloadRecordDao.listExpireTask();
		List<String> idList = new ArrayList<>();
		if(CollectionUtils.isNotEmpty(downloadRecordList)){
			for(DownloadRecord downloadRecord : downloadRecordList){
				idList.add(downloadRecord.getId());
			}
		}
		if(CollectionUtils.isNotEmpty(idList)){
			//设置超时任务的状态为error
			this.downloadTaskDao.updateTaskStatusError(idList);
		}
	}


}
